@@ -985,7 +985,15 @@ class Cast(ColumnElement):
         visitor.visit_cast(self)
     def _get_from_objects(self):
         return self.clause._get_from_objects()
-        
+    def _make_proxy(self, selectable, name=None):
+        if name is not None:
+            co = ColumnClause(name, selectable, type=self.type)
+            co.orig_set = self.orig_set
+            selectable.columns[name]= co
+            return co
+        else:
+            return self
+            
 class FunctionGenerator(object):
     """generates Function objects based on getattr calls"""
     def __init__(self, engine=None):
@@ -1210,7 +1218,7 @@ class ColumnClause(ColumnElement):
     def _bind_param(self, obj):
         return BindParamClause(self._label, obj, shortname = self.name, type=self.type)
     def _make_proxy(self, selectable, name = None):
-        c = ColumnClause(name or self.name, selectable, hidden=self.hidden)
+        c = ColumnClause(name or self.name, selectable, hidden=self.hidden, type=self.type)
         c.orig_set = self.orig_set
         if not self.hidden:
             selectable.columns[c.name] = c
@@ -1225,15 +1233,12 @@ class TableClause(FromClause):
         super(TableClause, self).__init__(name)
         self.name = self.fullname = name
         self._columns = util.OrderedProperties()
-        self._indexes = util.OrderedProperties()
         self._foreign_keys = []
         self._primary_key = []
         for c in columns:
             self.append_column(c)
         self._oid_column = ColumnClause('oid', self, hidden=True)
 
-    indexes = property(lambda s:s._indexes)
-
     def named_with_column(self):
         return True
     def append_column(self, c):
@@ -1250,16 +1255,11 @@ class TableClause(FromClause):
                 for ci in c.orig_set:
                     self._orig_cols[ci] = c
             return self._orig_cols
-    columns = property(lambda s:s._columns)
-    c = property(lambda s:s._columns)
-    primary_key = property(lambda s:s._primary_key)
-    foreign_keys = property(lambda s:s._foreign_keys)
     original_columns = property(_orig_columns)
 
     def _clear(self):
         """clears all attributes on this TableClause so that new items can be added again"""
         self.columns.clear()
-        self.indexes.clear()
         self.foreign_keys[:] = []
         self.primary_key[:] = []
         try:
@@ -1357,7 +1357,6 @@ class CompoundSelect(SelectBaseMixin, FromClause):
             col = column._make_proxy(self, name=column._label)
         else:
             col = column._make_proxy(self, name=column.name)
-        
         try:
             colset = self._col_map[col.name]
         except KeyError:
